#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import with_statement
from __future__ import unicode_literals

import pytest
import sys
from sets import Set
from dnscherry.auth.modLdap import Auth, CaFileDontExist
import cherrypy
import logging
import ldap

cfg = {
'auth.ldap.module': 'dnscherry.backend.ldap',
'auth.ldap.groupdn': 'ou=groups,dc=example,dc=org',
'auth.ldap.userdn': 'ou=People,dc=example,dc=org',
'auth.ldap.binddn': 'cn=dnscherry,dc=example,dc=org',
'auth.ldap.bindpassword': 'password',
'auth.ldap.uri': 'ldap://ldap.dnscherry.org:389',
'auth.ldap.ca': './tests/test_env/etc/dnscherry/TEST-cacert.pem',
'auth.ldap.starttls': 'off',
'auth.ldap.checkcert': 'off',
'auth.ldap.user.filter.tmpl': '(uid=%(login)s)',
'auth.ldap.group.filter.tmpl': '(member=%(userdn)s)',
'auth.ldap.dn_user_attr': 'uid',
'auth.ldap.group_attr.member': "%(dn)s",
'auth.ldap.timeout': 10,
}


def syslog_error(msg='', context='',
        severity=logging.INFO, traceback=False):
    pass

cherrypy.log.error = syslog_error
attr = ['shéll', 'shell', 'cn', 'uid', 'uidNumber', 'gidNumber', 'home', 'userPassword', 'givenName', 'email', 'sn']


class TestError(object):

    def testNominal(self):
        inv = Auth(cfg, cherrypy.log)
        return True

    def testConnectSSLNoCheck(self):
        cfg2 = cfg.copy()
        cfg2['uri'] = 'ldaps://ldap.dnscherry.org:636'
        cfg2['checkcert'] = 'off'
        inv = Auth(cfg2, cherrypy.log)
        ldap = inv._connect()
        ldap.simple_bind_s(inv.binddn, inv.bindpassword)

    def testConnect(self):
        inv = Auth(cfg, cherrypy.log)
        ldap = inv._connect()
        ldap.simple_bind_s(inv.binddn, inv.bindpassword)
        return True

    def testConnectSSL(self):
        cfg2 = cfg.copy()
        cfg2['uri'] = 'ldaps://ldap.dnscherry.org:636'
        cfg2['checkcert'] = 'on'
        inv = Auth(cfg2, cherrypy.log)
        ldap = inv._connect()
        ldap.simple_bind_s(inv.binddn, inv.bindpassword)

    def testLdapUnavaible(self):
        cfg2 = cfg.copy()
        cfg2['uri'] = 'ldaps://notaldap:636'
        cfg2['checkcert'] = 'on'
        inv = Auth(cfg2, cherrypy.log)
        try:
            ldapc = inv._connect()
            ldapc.simple_bind_s(inv.binddn, inv.bindpassword)
        except ldap.SERVER_DOWN as e:
            return

    def testMissingCA(self):
        cfg2 = cfg.copy()
        cfg2['uri'] = 'ldaps://ldap.dnscherry.org:636'
        cfg2['checkcert'] = 'on'
        cfg2['ca'] = './test/cfg/not_a_ca.crt'
        try:
            inv = Auth(cfg2, cherrypy.log)
            ldapc = inv._connect()
        except CaFileDontExist as e:
            return

    def testConnectSSLWrongCA(self):
        cfg2 = cfg.copy()
        cfg2['uri'] = 'ldaps://ldap.dnscherry.org:636'
        cfg2['checkcert'] = 'on'
        inv = Auth(cfg2, cherrypy.log)
        ldapc = inv._connect()
        try:
            ldapc.simple_bind_s(inv.binddn, inv.bindpassword)
        except ldap.SERVER_DOWN as e:
            assert e[0]['info'] == 'TLS: hostname does not match CN in peer certificate'

    def testConnectStartTLS(self):
        cfg2 = cfg.copy()
        cfg2['uri'] = 'ldap://ldap.dnscherry.org:390'
        cfg2['checkcert'] = 'off'
        cfg2['starttls'] = 'on'
        cfg2['ca'] = './test/cfg/ca.crt'
        inv = Auth(cfg2, cherrypy.log)
        ldapc = inv._connect()
        ldapc.simple_bind_s(inv.binddn, inv.bindpassword)

    def testAuthSuccess(self):
        inv = Auth(cfg, cherrypy.log)
        ret = inv.check_credentials('jwatson', 'passwordwatson')
        assert ret == True

    def testAuthFailure(self):
        inv = Auth(cfg, cherrypy.log)
        res = inv.check_credentials('notauser', 'password') or inv.check_credentials('jwatson', 'notapassword')
        assert res == False

    def testMissingParam(self):
        cfg2 = {}
        return True
        try:
            inv = Auth(cfg2, cherrypy.log)
        except MissingKey:
            return



# -*- coding: utf-8 -*-
# vim:set expandtab tabstop=4 shiftwidth=4:
# The MIT License (MIT)
# DnsCherry
# Copyright (c) 2014 Carpentier Pierre-Francois

import cherrypy
import dnscherry.auth


class Auth(dnscherry.auth.Auth):

    def __init__(self, config, logger=None):
        # no need for a logout button
        self.logout_button = False
        if 'auth.user_header_name' in config:
            self.user_header_name = config['auth.none.user_header_name']
        else:
            self.user_header_name = None

    def check_auth(self):
        if self.user_header_name is None:
            return 'unknown user'
        else:
            if self.user_header_name in cherrypy.request.headers:
                return cherrypy.request.headers[self.user_header_name]
            else:
                raise cherrypy.HTTPError(
                    "403 Forbidden",
                    "You are not allowed to access this resource."
                    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import with_statement
from __future__ import unicode_literals

import pytest
import sys
import subprocess
from tempfile import NamedTemporaryFile as tempfile
import re

from sets import Set
from dnscherry import DnsCherry

import cherrypy
from cherrypy.process import plugins, servers
from cherrypy import Application
import logging

cherrypy.session = {}


def loadconf(configfile, instance):
    app = cherrypy.tree.mount(instance, '/', configfile)
    cherrypy.config.update(configfile)
    instance.reload(app.config)

class HtmlValidationFailed(Exception):
    def __init__(self, out):
        self.errors = out

def htmlvalidator(page):
    f = tempfile()
    stdout = tempfile()
    f.write(page.encode("utf-8"))
    f.seek(0)
    ret = subprocess.call(['./tests/html_validator.py', '-h', f.name], stdout=stdout)
    stdout.seek(0)
    out = stdout.read()
    f.close()
    stdout.close()
    print(out)
    if not re.search(r'Error:.*', out) is None:
        raise HtmlValidationFailed(out)

class BadModule():
    pass


# monkey patching cherrypy to disable config interpolation
def new_as_dict(self, raw=True, vars=None):
    """Convert an INI file to a dictionary"""
    # Load INI file into a dict
    result = {}
    for section in self.sections():
        if section not in result:
            result[section] = {}
        for option in self.options(section):
            value = self.get(section, option, raw=raw, vars=vars)
            try:
                value = cherrypy.lib.reprconf.unrepr(value)
            except Exception:
                x = sys.exc_info()[1]
                msg = ("Config error in section: %r, option: %r, "
                       "value: %r. Config values must be valid Python." %
                       (section, option, value))
                raise ValueError(msg, x.__class__.__name__, x.args)
            result[section][option] = value
    return result
cherrypy.lib.reprconf.Parser.as_dict = new_as_dict

class TestError(object):

    def testNominal(self):
        app = DnsCherry()
        loadconf('./tests/cfg/dnscherry.ini', app)
        return True




# -*- coding: utf-8 -*-
# vim:set expandtab tabstop=4 shiftwidth=4:
# The MIT License (MIT)
# DnsCherry
# Copyright (c) 2014 Carpentier Pierre-Francois

import cherrypy
import ldap
import dnscherry.auth
import logging

SESSION_KEY = '_cp_username'


class CaFileDontExist(Exception):
    def __init__(self, cafile):
        self.cafile = cafile
        self.log = "CA file %(cafile)s does not exist" % {'cafile': cafile}


class Auth(dnscherry.auth.Auth):

    def __init__(self, config, logger=None):

        self.logger = logger
        self.logout_button = True
        self.userdn = self._get_param('auth.ldap.userdn', config)
        self.user_filter_tmpl = self._get_param(
            'auth.ldap.user.filter.tmpl',
            config
            )
        self.groupdn = self._get_param('auth.ldap.groupdn', config, False)

        if self.groupdn:
            self.group_filter_tmpl = self._get_param(
                'auth.ldap.group.filter.tmpl',
                config
                )

        self.binddn = self._get_param('auth.ldap.binddn', config)
        self.bindpassword = self._get_param('auth.ldap.bindpassword', config)
        self.uri = self._get_param('auth.ldap.uri', config)
        self.ca = self._get_param('auth.ldap.ca', config, False)
        self.starttls = self._get_param('auth.ldap.starttls', config, 'off')
        self.checkcert = self._get_param('auth.ldap.checkcert', config, 'on')
        self.timeout = self._get_param('auth.ldap.timeout', config, 1)

    def _connect(self):
        ldap_client = ldap.initialize(self.uri)
        ldap_client.set_option(ldap.OPT_REFERRALS, 0)
        ldap.set_option(ldap.OPT_TIMEOUT, self.timeout)

        if self.starttls == 'on':
            ldap.set_option(ldap.OPT_X_TLS_DEMAND, True)
        else:
            ldap.set_option(ldap.OPT_X_TLS_DEMAND, False)
        # set the CA file if declared and if necessary
        if self.ca and self.checkcert == 'on':
            # check if the CA file actually exists
            if os.path.isfile(self.ca):
                ldap.set_option(ldap.OPT_X_TLS_CACERTFILE, self.ca)
            else:
                raise CaFileDontExist(self.ca)
        if self.checkcert == 'off':
            # this is dark magic
            # remove any of these two lines and it doesn't work
            ldap.set_option(ldap.OPT_X_TLS_REQUIRE_CERT, ldap.OPT_X_TLS_NEVER)
            ldap_client.set_option(
                ldap.OPT_X_TLS_REQUIRE_CERT,
                ldap.OPT_X_TLS_NEVER
                )
        else:
            # this is even darker magic
            ldap_client.set_option(
                ldap.OPT_X_TLS_REQUIRE_CERT,
                ldap.OPT_X_TLS_DEMAND
                )
            # it doesn't make sense to set it to never
            # (== don't check certifate)
            # but it only works with this option...
            # ... and it checks the certificat
            # (I've lost my sanity over this)
            ldap.set_option(
                ldap.OPT_X_TLS_REQUIRE_CERT,
                ldap.OPT_X_TLS_NEVER
                )

        if self.starttls == 'on':
            try:
                ldap_client.start_tls_s()
            except ldap.OPERATIONS_ERROR:
                self._logger(
                    logging.ERROR,
                    "cannot use starttls with ldaps:// uri (uri: " +
                    self.uri + ")"
                )
                raise cherrypy.HTTPError(
                        "500",
                        "Configuration Error, contact administrator"
                        )
        try:
            ldap_client.simple_bind_s(self.binddn, self.bindpassword)
        except ldap.INVALID_CREDENTIALS:
            self._logger(
                    logging.ERROR,
                    "Configuration error, wrong credentials, "
                    "unable to connect to ldap with '" + self.binddn + "'"
                )
            raise cherrypy.HTTPError(
                    "500",
                    "Configuration Error, contact administrator"
                    )
        except ldap.SERVER_DOWN:
            self._logger(
                    logging.ERROR,
                    "Unable to contact ldap server '" +
                    self.uri +
                    "', check 'auth.ldap.uri' and ssl/tls configuration",
                )
            raise cherrypy.HTTPError(
                    "500",
                    "Configuration Error, contact administrator"
                    )
        return ldap_client

    def check_credentials(self, username, password):

        ldap_client = self._connect()

        user_filter = self.user_filter_tmpl % {
            'login': username
            }

        r = ldap_client.search_s(
                self.userdn,
                ldap.SCOPE_SUBTREE,
                user_filter
                )
        if len(r) == 0:
            ldap_client.unbind_s()
            return False

        dn_entry = r[0][0]

        try:
            ldap_auth = ldap.initialize(self.uri)
            if self.starttls == 'on':
                ldap_auth.start_tls_s()
            ldap_auth.set_option(ldap.OPT_REFERRALS, 0)
            ldap_auth.simple_bind_s(dn_entry, password)
            ldap_auth.unbind_s()
        except ldap.INVALID_CREDENTIALS:
            ldap_client.unbind_s()
            return False

        if self.groupdn:
            group_filter = self.group_filter_tmpl % {
                    'userdn': dn_entry
                    }
            r = ldap_client.search_s(
                    self.groupdn,
                    ldap.SCOPE_SUBTREE,
                    group_filter
                    )
            if len(r) == 0:
                ldap_client.unbind_s()
                return False

        ldap_client.unbind_s()
        return True
